<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Application\EventListener;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\ReferralAlreadyAcceptedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\ReferralNotFoundForInviteeEmail;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\AcceptInvite\AcceptInvite;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\AcceptInvite\AcceptInviteInput;
use Psr\Log\LoggerInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\GenericEvent;

class CustomerPostRegisterEventListener
{
    /** @var AcceptInvite */
    private $acceptInvite;

    /** @var ReferralRepositoryInterface */
    private $referralRepository;

    /** @var EventDispatcherInterface */
    private $eventDispatcher;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        AcceptInvite $acceptInvite,
        ReferralRepositoryInterface $referralRepository,
        EventDispatcherInterface $eventDispatcher,
        LoggerInterface $logger
    ) {
        $this->acceptInvite = $acceptInvite;
        $this->referralRepository = $referralRepository;
        $this->eventDispatcher = $eventDispatcher;
        $this->logger = $logger;
    }

    public function __invoke(GenericEvent $event): void
    {
        $customer = $event->getSubject();
        if (!$customer instanceof CustomerInterface || !$customer->getUser()) {
            return;
        }

        try {
            $presenter = new CustomerPostRegisterEventListenerPresenter();
            ($this->acceptInvite)(new AcceptInviteInput($customer->getUser()), $presenter);

            $events = [
                [new GenericEvent($presenter->getReferral()), 'dedi_sylius_customer_referral_plugin.referral_accepted'],
                [new GenericEvent($presenter->getInviterCustomer()), 'dedi_sylius_customer_referral_plugin.inviter_has_new_registered_invitee'],
                [new GenericEvent($presenter->getInviteeCustomer()), 'dedi_sylius_customer_referral_plugin.invitee_accept_invite'],
            ];

            foreach ($events as $event) {
                call_user_func([$this->eventDispatcher, 'dispatch'], ...$event);
            }
        } catch (ReferralNotFoundForInviteeEmail $e) {
            // do nothing on customer post register if he has not been invited by an other user
        } catch (ReferralAlreadyAcceptedException $e) {
            $this->logger->warning('[dedi_customer_referral_plugin] Referral already accepted', ['exception' => $e]);
        } catch (InviteeAlreadyInvitedException $e) {
            $this->logger->warning('[dedi_customer_referral_plugin] Invitee already invited', ['exception' => $e]);
        }
    }
}
